"use strict";

this.author      = "spara"; 
this.copyright   = "2015 spara"; 
this.description = "asteroid respawner and multiplier"; 
this.name        = "asteroid_tweaks";

//how far from the player to spawn?
this.$minimumSpawnDistance = 500E3;

this.startUp = function() {
	//asteroid respawn
	if (!missionVariables.asteroid_tweaks_respawn) {
		this.$respawn = false;
	}
	else this.$respawn = true;
	
	//asteroid multiplier
	if (!missionVariables.asteroid_tweaks_multiplier) {
		missionVariables.asteroid_tweaks_multiplier = 1;
	}
	this.$asteroidMultiplier = missionVariables.asteroid_tweaks_multiplier;
	
	//main station configuration interface
	if (!missionVariables.asteroid_tweaks_config_main) {
		this.$configMain = true;
	}
	else this.$configMain = false;
}

//multiply the number of asteroids when populating
this.systemWillPopulate = function (){
	var addition = this.$asteroidMultiplier - 1;
	if (addition > 0) {
		system.setPopulator("moar_rocks", {
			callback: function(pos) {
				var asteroids = system.shipsWithPrimaryRole("asteroid");
				for (var i = 0; i < asteroids.length; i++) {
					var position = asteroids[i].position;
					system.addShips("asteroid", addition, position);
				};
			}.bind(this),
			location: "INNER_SYSTEM_OFFPLANE"
		});
	}
}

//asteroids added outside the populate phase are ignored from the total count.
this.shipExitedWitchspace = function() {
	if (this.$respawn) {
		this.$asteroidsTotal = system.countShipsWithPrimaryRole("asteroid");
	}
}

this.startUpComplete = function() {
	this.shipWillDockWithStation(player.ship.dockedStation);
	this.shipExitedWitchspace();
}

//replace destroyed asteroids when docking.
this.shipDockedWithStation = function(station) {
	function $findAsteroids(entity){
		if (entity.primaryRole === "asteroid") {
			if (entity.position.distanceTo(player.ship.position) > this.$minimumSpawnDistance) {
				return true;
			}
		}
		return false;
	}
	function $findCloseAsteroids(entity){
		if (entity.primaryRole === "asteroid") {
			if (entity.position.distanceTo(player.ship.position) < 25000) {
				return true;
			}
		}
		return false;
	}
	if (this.$respawn) {
		var difference = this.$asteroidsTotal - system.countShipsWithPrimaryRole("asteroid");
		if (difference > 0) {
			var ws = worldScripts["miningcontracts"];
			if (station.primaryRole.indexOf("rockhermit") !== -1 && ws && ws.MC_active) {
				var closeAsteroids = system.filteredEntities(this, $findAsteroids, player.ship);
				if (closeAsteroids.length > 0) {
					if (difference > 3) {
						system.addShips("asteroid", 3, closeAsteroids[0].position);
						difference =- 3;
					}
					else {
						system.addShips("asteroid", difference, closeAsteroids[0].position);
						difference = 0;
					}
				}
			}
		}
		if (difference > 0) {
			//report asteroids over min distance away from player
			var asteroids = system.filteredEntities(this, $findAsteroids, player.ship);
			//Add field only if it can be added around an asteroid far away enough
			if (asteroids.length > 0) {
				var spawnPosition = asteroids[Math.floor(Math.random(asteroids.length))].position;
				//add asteroids as a cloud
				system.addShips("asteroid", difference, spawnPosition);
			}
			//In case there's no suitable asteroid to be used as a spawn position, spread 'em around the player
			else {
				for (var i = 0; i < difference; i++) {
					var spawnPosition = player.ship.position.add(Vector3D.randomDirection(this.$minimumSpawnDistance)).multiply(1 + Math.random());
					system.addShips("asteroid", 1, spawnPosition);
				}
			}
		}
	}
}

this.shipWillDockWithStation = function(station) {
	if (station.primaryRole.indexOf("rockhermit") !== -1 || (station.isMainStation && this.$configMain)) {
		station.setInterface("asteroid_tweaks",{
			title: "Asteroid tweaks",
			category: "Config",
			summary: "Configure asteroid tweaks.",
			callback: this.$showAsteroidConfig.bind(this)
		});
	}
}

this.$showAsteroidConfig = function() {
	var options = {
		"1_X" : "1x",
		"2_X" : "2x",
		"3_X" : "3x",
		"4_X" : "4x",
		"5_X" : "5x"
	};
	mission.runScreen({
		title: "Asteroid Tweaks",
		message: "Do you want moar asteroids? Multiply the number of asteroids the game spawns when a system is being created. Note though, more asteroids means more stress for your computer. Current multiplier is " + this.$asteroidMultiplier + ".",
		choices: options
	},
	function (choice) {
		this.$asteroidMultiplier = choice.split('_')[0];
		missionVariables.asteroid_tweaks_multiplier = this.$asteroidMultiplier;
		this.$showRespawnConfig();
	});
}

this.$showRespawnConfig = function() {
	var options = {
		"1_NO" : "No, don't respawn asteroids",
		"2_YES" : "Yes, respawn asteroids"
	};
	mission.runScreen({
		title: "Asteroid Tweaks",
		message: "In the default game, it's possible to empty a system from asteroids by destroying them all. This setting makes the game respawn destroyed asteroids as a new asteroid field somewhere in system when the player docks. Current asteroid respawn setting is: " + this.$respawn + ".",
		choices: options
	},
	function (choice) {
		if (choice === "1_NO" && this.$respawn) {
			this.$respawn = false;
			delete missionVariables.asteroid_tweaks_respawn;
		}
		else if (choice === "2_YES" && !this.$respawn) {
			this.$respawn = true;
			this.shipExitedWitchspace();
			missionVariables.asteroid_tweaks_respawn = 1;
		}
		this.$showMainConfig();
	});
}

this.$showMainConfig = function() {
	var options = {
		"1_YES" : "Yes, show it at the main station",
		"2_NO" : "No, don't show it at the main station"
	};
	mission.runScreen({
		title: "Asteroid Tweaks",
		message: "Show this config interface at the main station interfaces. Interface is always available at any rock hermit.\n\nDon't worry, the settings you have chosen will remain active even if you decide to hide the interface from the main station.",
		choices: options
	},
	function (choice) {
		if (choice === "1_YES" && !this.$configMain) {
			this.$configMain = true;
			delete missionVariables.asteroid_tweaks_config_main;
		}
		else if (choice === "2_NO" && this.$configMain){
			this.$configMain = false;
			missionVariables.asteroid_tweaks_config_main = 1;
			system.mainStation.setInterface("asteroid_tweaks",null);
		}
	});
}
